@@ -24,10 +24,11 @@ import cgi
 import json
 import os
 import time
+import traceback
 
 import pkg_resources
 
-from muninplot.data import get_info, get_resolutions, get_values
+from muninplot.data import get_info, get_values
 
 
 # The directory that contains the JSON files that describe the dashboards
@@ -80,10 +81,13 @@ def list_dashboards(environ, start_response):
         for filename in sorted(os.listdir(dashboards_dir)):
             filename = os.path.join(dashboards_dir, filename)
             if filename.endswith('.json') and os.path.isfile(filename):
-                with open(filename, 'rt') as f:
-                    dashboard = json.load(f)
-                    dashboard.setdefault('name', os.path.basename(filename)[:-5])
-                    dashboards[dashboard['name']] = dashboard
+                try:
+                    with open(filename, 'rt') as f:
+                        dashboard = json.load(f)
+                        dashboard.setdefault('name', os.path.basename(filename)[:-5])
+                        dashboards[dashboard['name']] = dashboard
+                except Exception:
+                    traceback.print_exc(file=environ['wsgi.errors'])
     start_response('200 OK', [
         ('Content-Type', 'application/json')])
     return [json.dumps(dashboards, indent=2, sort_keys=True).encode('utf-8')]
@@ -108,7 +112,7 @@ def _parse_timestamp(timestamp):
         '%Y-%m-%d')
     for fmt in formats:
         try:
-            return time.mktime(time.strptime(timestamp, fmt))
+            return int(time.mktime(time.strptime(timestamp, fmt)))
         except ValueError:
             pass
     raise ValueError('time data %r does not match any known format' % timestamp)
@@ -118,36 +122,19 @@ def get_data(environ, start_response):
     """Return a data series for the graph as CSV."""
     path = environ.get('PATH_INFO', '').lstrip('/')
     _, group, host, graph = path.split('/')
-    last_update, resolutions = get_resolutions(group, host, graph)
     parameters = cgi.parse_qs(environ.get('QUERY_STRING', ''))
     # get the time range to fetch the data for
     end = parameters.get('end')
-    end = _parse_timestamp(end[0]) if end else last_update
+    end = _parse_timestamp(end[0]) if end else time.time()
     start = parameters.get('start')
     start = _parse_timestamp(start[0]) if start else end - 24 * 60 * 60 * 7
-    # calculate the minimum resolution that we want
-    resolution = min((
-        parameters.get('resolution', (end - start) / 5000),
-        resolutions[-1][0]))
-    # find the resolution that contains the whole data set
-    for res, rows in resolutions:
-        if res >= resolution:
-            s = max((last_update - res * rows, start))
-            e = min((last_update, end))
-            if e > s:
-                resolution = res
-                break
-    values = get_values(group, host, graph, start, end, resolution)
     # return the values as CSV
     start_response('200 OK', [
         ('Content-Type', 'text/csv')])
-    if values:
-        keys = (x for x in values[0].keys() if x != 'remove')
-        keys = ['time'] + sorted((k for k in keys if k != 'time'), key=_field_key)
-        yield ('%s\n' % (','.join(keys))).encode('utf-8')
-        for value in values:
-            value['time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(value['time']))
-            yield ('%s\n' % (','.join(str(value.get(key, '')) for key in keys))).encode('utf-8')
+    for values in get_values(group, host, graph, start, end):
+        if not isinstance(values[0], str):
+            values[0] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(values[0]))
+        yield ('%s\n' % (','.join(str('' if value is None else value) for value in values))).encode('utf-8')
 
 
 def application(environ, start_response):
