@@ -86,9 +86,11 @@ $(document).ready(function () {
   // initialise the date range picker
   $('#reportrange').daterangepicker({
     locale: {
-      format: 'YYYY-MM-DD HH:mm'
+      format: 'YYYY-MM-DD HH:mm',
+      firstDay: 1
     },
     opens: 'left',
+    showISOWeekNumbers: true,
     timePicker: true,
     timePickerIncrement: 10,
     timePicker24Hour: true,
@@ -103,6 +105,16 @@ $(document).ready(function () {
     }
   }, setDateRange)
 
+  // update the ranges when the date range picker is opened
+  $('#reportrange').on('show.daterangepicker', function (ev, picker) {
+    picker.ranges = {
+      Today: [moment().subtract(1, 'days').round10Minutes(), moment().add(1, 'hour').round10Minutes('ceil')],
+      'This week': [moment().subtract(6, 'days').startOf('day'), moment().endOf('day').round10Minutes('ceil')],
+      'This month': [moment().subtract(32, 'days').startOf('day'), moment().endOf('day').round10Minutes('ceil')],
+      'This year': [moment().subtract(365, 'days').startOf('month'), moment().endOf('month').round10Minutes('ceil')]
+    }
+  })
+
   try {
     // restore the previously saved date range
     const data = JSON.parse(localStorage.getItem('dateRange'))
@@ -119,6 +131,13 @@ $(document).ready(function () {
     '#80c9ff', '#ffc080', '#ffe680', '#aa80ff', '#ee00cc', '#ff8080',
     '#666600', '#ffbfff', '#00ffcc', '#cc6699', '#999900']
 
+  function getColor(value) {
+    if (value.startsWith('COLOUR')) {
+      return defaultColors[parseInt(value.substring(6)) % defaultColors.length]
+    }
+    return '#' + value
+  }
+
   const baseLayout = {
     margin: {l: 48, t: 0, r: 8, b: 32},
     autosize: true,
@@ -372,7 +391,7 @@ $(document).ready(function () {
     let stackgroup = 0
     for (let i = 0; i < plot.graph.fields.length; i++) {
       const field = plot.graph.fields[i]
-      const color = field.colour ? '#' + field.colour : defaultColors[i % defaultColors.length]
+      const color = field.colour ? getColor(field.colour) : defaultColors[i % defaultColors.length]
       if (field.draw === 'AREA' || field.draw === 'STACK' || field.draw === 'AREASTACK') {
         if (!field.draw.match(/STACK/) && (!plot.graph.fields[i + 1] || plot.graph.fields[i + 1].draw.match(/STACK/))) {
           stackgroup += 1
@@ -434,7 +453,7 @@ $(document).ready(function () {
     traces.slice().reverse().forEach(function (trace) {
       if (trace.showlegend !== false) {
         const legendrow = $('<tr></tr>')
-        legendrow.append($('<td style="width: 30px;"><svg height="10" width="20"><line x1="0" y1="5" x2="20" y2="5"></svg></td>'))
+        legendrow.append($('<td style="width: 20px;"><svg height="10" width="10"><line x1="0" y1="5" x2="10" y2="5"></svg></td>'))
         legendrow.append($('<td><span></span></td>'))
         legendrow.append($('<td></td>'))
         legendrow.append($('<td></td>'))
