@@ -9,7 +9,8 @@ module.exports = (env, options) => {
     output: {
       filename: 'munin-plot.[contenthash].js',
       path: path.resolve(__dirname, 'muninplot/static'),
-      publicPath: ''
+      publicPath: '',
+      assetModuleFilename: '[name][ext]'
     },
     optimization: {
       minimize: options.mode === 'production'
@@ -23,8 +24,10 @@ module.exports = (env, options) => {
       }),
       new webpack.ProvidePlugin({
         $: 'jquery',
-        jQuery: 'jquery',
         Plotly: 'plotly.js/dist/plotly-basic',
+        bootstrap: 'bootstrap',
+        d3: 'd3',
+        jQuery: 'jquery',
         moment: 'moment'
       })
     ],
@@ -52,15 +55,7 @@ module.exports = (env, options) => {
         },
         {
           test: /\.(ico|png)$/,
-          use: [
-            {
-              loader: 'file-loader',
-              options: {
-                name: '[name].[ext]',
-                esModule: false
-              }
-            }
-          ]
+          type: 'asset/resource'
         },
         {
           test: /\.(html)$/,
